#include "courselib.h"
#include <cmath>
#include "TH1.h"
#include "TFile.h" 
#include "TCanvas.h"
#include <TApplication.h>
#include <TStyle.h>
#include <TROOT.h>
#include <iostream>
#include <string>
#include <iterator>
#include <iostream>
#include <cstdlib>

using namespace std;

/*
	Monte Carlo integration.
	We calculate the x-section for p p -> gamma* (Z0)--> l+l- + X
      and plot the distribution in pt and eta

	The random number generator is RANLUX by M. Luescher.
      M. Lscher, A portable high-quality random number generator 
      for lattice field theory simulations, 
      Computer Physics Communications 79 (1994) 100  
	http://luscher.web.cern.ch/luscher/ranlux/index.html      


*/

typedef struct {
    double e;
    double px,py,pz;
    double m;
} PVEC;
#define DOTPR(p1,p2) (p1.e*p2.e-p1.px*p2.px-p1.py*p2.py-p1.pz*p2.pz)


double sigma(double m2, double sh, double th)
{	
	static const double pi = 3.14159265358979323848;
	double aem=1./137.;
	double result,uh;
      result = 0;
      uh = m2 -sh -th;
// here one can insert the O(alphas) ME.
// for our purpose we use only the lowest order qq-> gamma*
// dsigma/dt (qq -> gamma)
	result = 4.*pi*pi*aem/3.;
	return(result); 
}

double gauss ( double mean, double sigma )
{	
	int n1, npoints = 12;
	double rvec[12];
      double result, r;
      result = 0.;
/*
      here generate kt2 from a gauss distribution
      with mean and sigma given
      return the kt2 in value result
*/     
      return(result);
}

void getpdf(double xmin, double q2,double& weightx, double& x, double& kx, double& ky, double& kt2 )
{
		static const double pi = 3.14159265358979323848;
		double xmax = 1,pdf,q;
            double phi;
      	double eq ;
		double rvec[2];
      	ranlxd(rvec,2);            
/*
		generate here x according to a 1/x distribution
            give a weight according a simple pdf ala
            xg(x) = 3(1-x)^5. 
// 		note, we want here the pdf, not x*pdf.
            
            caldulate kt2 from the gauss distribution with
            mean = 0 and width = 0,7  in kt2 
            use a random azimuthal angle phi to get k_x,k_y
            
            give back weightx, kx, ky and kt2
            
            
*/            
            x=0.01; 
            weightx =1 ;
      	kt2 = gauss(0,0.7);
      	phi = 2*pi*rvec[0];
            kx = sqrt(kt2)*cos(phi);
            ky = sqrt(kt2)*sin(phi);
//            cout << "in getx:  x " << x << " kx " << kx <<" ky " << ky<<endl; 

}

int main(int argc,char **argv)
{
      TApplication* gMyRootApp = new TApplication("My ROOT Application", &argc, argv);
	int n1, npoints = 1.E6, nacc=0;
 	double x1,x2;
      double x1min=0.00001, x2min=0.00001;
      double Hx1x2,Fact_gamma, mass,sh=0,th=0,q2;
      double s=4*3500.*3500.,stest;
      double kt2, kt21,kt22;
      double pt2, eta;
      double weightx1, weightx2, eq2;
      double sum0, sum00, f, ff, sigma2, error;
      double kx,ky;


	static const double pi = 3.14159265358979323848;
	double aem=1./137.;
	static const double gev2nb=0.3893857E+6;
    
      PVEC pin1,pin2, p_a, p_b, pout;
      
//               TH1F("label","title",nr of bins, xlow,xhigh )
      TH1F *histo1 = new TH1F("x1","x1",100, -5, 0.);
      TH1F *histo10 = new TH1F(*histo1); 
      double binwidth1=5./100.;
      TH1F *histo2 = new TH1F("x2","x2",100, -5, 0.);
      TH1F *histo20 = new TH1F(*histo2); 
      TH1F *histo3 = new TH1F("kt1 ","kt1 ",100, 0, 10.);
      TH1F *histo4 = new TH1F("kt2 ","kt2 ",100, 0, 10.);
      TH1F *histo5 = new TH1F("pt ","pt ",100, 0, 10.);
      TH1F *histo6 = new TH1F("eta ","eta",100, -8, 8.);
      TH1F *histo7 = new TH1F("Mass ","mass",50, 50., 150.);
      
      // initialise random number generator: rlxd_init( luxory level, seed )
      rlxd_init(2,32767);

      sum0 = sum00 = 0;
      
      // define incoming protons      
      pin1.px = 0;
      pin1.py = 0;
      pin1.pz = sqrt(s)/2;
      pin1.e = sqrt(s)/2;
      
      pin2.px = 0;
      pin2.py = 0;
      pin2.pz = -sqrt(s)/2;
      pin2.e = sqrt(s)/2;
      
      stest = sqrt(2*DOTPR(pin1,pin2));
      
      cout << " sqrt(s) = " << stest << endl;
      q2 = 10000.;
      double mass_min = 70.;
      double mass_max = 110.;
      eq2 = 2. *( 1. + 4. + 1. + 4.)/9. ;
	for ( n1 = 0; n1 < npoints; n1++ ) {
/*
  this is using importance sampling
  generate starting distribution in x and kt
  for first incoming parton
  calculate px,py,pz, E and kt2
  use the four-vector struct declaration 
  
  generate starting distribution in x and kt
  for second incoming parton
  calculate px,py,pz, E and kt2
  use the four-vector struct declaration 
  
  calculate the mass of the resulting system (pa+pb)^2
  calculate the pdfs at x1,q2 and x2,q2, with q2=10000
  
  calculate the cross section 
  this is using importance sampling

  Fact_gamma = factor gamma*->l^+l^-
  Fact_gamma = aem/3./pi/mass/mass;
  calculate eta of the DY pair
  investigate DY pairs in the range 70 < m < 110 GeV
  xsec=pdf(x1,q2)*pdf(x2,q2)*sigma*gev2nb
  the pdf should be included in the weight for x
  
  calculate the integral and its uncertainty
  
  plot the distribution in x1,x2 versus log10(x)
  which Jacobian is needed to get the proper distribution ?

*/         
         // plot dxg(x)/dlogx *Jacobian, Jacobian dlogx/dx = 1/x
         histo1->Fill(log10(x1),weightx1/2.3026);
         histo2->Fill(log10(x2),weightx2/2.3026);
         histo3->Fill(sqrt(kt21),weightx1);
         histo4->Fill(sqrt(kt22),weightx2);


      }
//                                  

      sum0 = sum0/npoints;
      sum00 = sum00/npoints;
      sigma2 = sum00 - sum0*sum0 ;
      error = sqrt(sigma2/npoints) ;
      cout<< " integral for DY xsection is [nb]: " << sum0<< " +/- " << error<< endl;
//    write histogramm out to file
      TFile file("output-example14.root","RECREATE");
//general root settings 
      gROOT->Reset();
      gROOT->SetStyle("Plain");
      gStyle->SetPadTickY(1); // ticks at right side
      gStyle->SetOptStat(0); // get rid of statistics box
      TCanvas *c = new TCanvas("ctest", "" ,0, 0, 500, 500);
// divide the canvas in 1 parts in x and 1 in y
      c -> Divide(3,3);
      c -> cd(1);
      a=1./npoints/binwidth1; 
      b=0; 
      histo10->Add(histo1,histo1,a,b); 
      histo10 -> Draw();
      c -> cd(2);
      a=1./npoints/binwidth1; 
      b=0; 
      histo20->Add(histo2,histo2,a,b); 
      histo20 -> Draw();
      c -> cd(3);
      histo3 -> Draw();
      c -> cd(4);
      histo4 -> Draw();
      c -> cd(5);
      histo5 -> Draw();
      c -> cd(6);
      histo6 -> Draw();
      c -> cd(7);
      histo7 -> Draw();
      c-> Draw();
      c->WaitPrimitive();
      c-> Print("example8.pdf");
      gMyRootApp->SetReturnFromRun(true);
      
      histo1->Write();
      histo2->Write();
      histo3->Write();
      histo4->Write();
      histo5->Write();
      histo6->Write();
      histo7->Write();
      file.Close();

	return(0);
}
